
OAMINDEX:	; OAM index for one tile sprites. 
db $40,$44,$48,$4C,$50,$54,$58,$5C,$60,$64,$68,$6C,$80,$84,$88,$8C,$B0,$B4,$B8,$BC


SHOTZO_KILL:
	LDA #$00	; kill self
	STA $1892,y
	RTS		; and return

Main:	; shotzo bullet
	JSR SUB_DRAWINFO; gets positions relative to screen border
			; 04 = y pos
			; 05 = x pos
			; 06 = x pos high bit
			; x register = 0 if onscreen, 1 if off above/below, 2 of off left/right
			; 00-01 = y pos (level)
			; 02-03 = x pos (level)
	CPX #$00
	BNE SHOTZO_KILL
	

	PHY		; push Y
	LDA OAMINDEX,y	; get OAM index
	TAY		; stick in Y

	LDA $14		; frame timer
	AND #$0F	; clear high nybble
	LSR		; / 2
	TAX		; stick in X

	LDA $05		; get x position relative to screen border
	CLC
	ADC SHOTZO_MOVEMENTX,x	; add in x offset
	STA $0200,y		; OAM

	LDA SHOTZO_MOVEMENTX,x	; if offset is negative..
	BMI SHOTZO_1		; skip
	LDA $06			; if $02 is already set... 
	BNE SHOTZO_1		; skip
	LDA #$00		;
	ADC #$00		; set high x according to carry flag, otherwise
	TSB $06
SHOTZO_1:

	LDA $04		; y pos
	CLC
	ADC SHOTZO_MOVEMENTY,x
	STA $0201,y	; OAM

	LDA #$20	; tile number
	STA $0202,Y	; OAM

	LDA #$35	; full priority, palette A, second graphics page
	STA $0203,y	; OAM

	TYA
	LSR
	LSR
	TAY
	LDA #$02	; set tile to be 16x16
	ORA $06		; add in high x bit
	STA $0420,Y
	PLY		; pull y back

	LDA $9D			; sprites frozen flag
	BNE SHOTZO_RETURN	; don't move or do collision if set

	JSR SUB_16X16_COLLISION2	; check collision. carry clear if contact
	BCS SHOTZO_0
	PHY
	JSL $00F5B7			; Hurt Mario if sprite is interacting.
	PLY
SHOTZO_0:

	JSR SUB_SPEEDS	; update based on speed values

SHOTZO_RETURN:
	RTS		; return

SHOTZO_MOVEMENTX:
	db $FF,$00
SHOTZO_MOVEMENTY:
	db $01,$02,$01,$00,$FF,$FE,$FF,$00



SUB_DRAWINFO:
	LDA $1E02,y	; y pos (low)
	STA $00		; store to scratch
	LDA $1E2A,y	; y pos (high)
	STA $01		; 
	LDA $1E16,y	; x pos (low)
	STA $02		;
	LDA $1E3E,y	; x pos (high)
	STA $03

	LDX #$01
	REP #$20	; 16 bit A
	LDA $00		; sprite y
	SEC
	SBC $1C		; layer 1 y
	BPL DRAW3
	CMP #$FFE0
	BCC OFFSCREEN	; return as offscreen if offscreen
	BRA DRAW4
DRAW3:
	CMP #$00E0
	BCS OFFSCREEN
DRAW4:
	STA $04		; store to scratch

	LDX #$02
	LDA $02		; sprite x
	SEC
	SBC $1A		; layer 1 x
	BPL DRAW1
	CMP #$FFE0
	BCC OFFSCREEN	; return as offscreen if offscreen
	BRA DRAW2	; otherwise continue
DRAW1:
	CMP #$0100
	BCS OFFSCREEN
DRAW2:
	AND #$01FF
	STA $05		; store to scratch as well
	
	SEP #$20	; 8 bit A
	LDX #$00	; return as onscreen

	RTS

OFFSCREEN:
	SEP #$20	; 8 bit A
	RTS		; 


SUB_SPEEDS:		
	LDA $1E52,y	; sprite y speed
	ASL
	ASL
	ASL
	ASL
	CLC
	ADC $1E7A,y	; accumulating fraction bits for y speed
	STA $1E7A,y	; set it
	PHP
	LDA $1E52,y	; sprite y speed
	LSR
	LSR
	LSR
	LSR
	CMP #$08
	LDX #$00
	BCC SPEED_1
	ORA #$F0
	DEX
SPEED_1:
	PLP
	ADC $1E02,y	; sprite y (low)
	STA $1E02,y	; set it
	TXA
	ADC $1E2A,y	; sprite y (high)
	STA $1E2A,y	; set it

	; do it all again for x speeds

	LDA $1E66,y	; sprite x speed
	ASL
	ASL
	ASL
	ASL
	CLC
	ADC $1E8E,y	; accumulating fraction bits for x speed
	STA $1E8E,y	; set it
	PHP
	LDA $1E66,y	; sprite x speed
	LSR
	LSR
	LSR
	LSR
	CMP #$08
	LDX #$00
	BCC SPEED_2
	ORA #$F0
	DEX
SPEED_2:
	PLP
	ADC $1E16,y	; sprite x (low)
	STA $1E16,y	; set it
	TXA
	ADC $1E3E,y	; sprite x (high)
	STA $1E3E,y	; set it

	RTS		; return

SUB_16X16_COLLISION2:		; checks collision for 16x16 cluster sprites
				; this one actually takes into account screen number
	PHX			; push x

	LDA $1E16,y		; sprite x (low)
	STA $00			; store to scratch
	LDA $1E3E,y		; sprite x (high)
	STA $01			; store to scratch
	LDA $1E02,y		; sprite y (low)
	STA $02
	LDA $1E2A,y		; sprite y (high)
	STA $03

	REP #$20		; 16 bit A
	LDA $94				;
	SEC				; checks if mario's in range horizontally
	SBC $00				;
	CLC				;
	ADC #$000A			;
	CMP #$0014			;
	SEP #$20		; 8 bit A
	BCS NO_CONTACT2			; return if not
	LDA #$18
	LDX $73				; mario is ducking flag
	BNE CONTACT_CONT2		; collision box = 10 if so
	LDX $19				; mario powerup
	BEQ CONTACT_CONT2		; collision box = 10 if small
	LDA #$24			; else collision box = 20

CONTACT_CONT2:
	STA $00				; store to scratch
	STZ $01				; set that to 00
	REP #$20		; 16 bit A
	LDA $96				; mario y
	SEC
	SBC $02			; sprite y
	CLC
	ADC #$001C			; offset by 10 (so as to not have overflow issues; was offset earlier too
	CMP $00
	SEP #$20		; 8 bit A
		; carry = clear (positive return)
		; carry = set (negative return)
NO_CONTACT2:
	PLX		; pull x

	RTS